/*
 * bg.c -- background texturing.
 *
 * (c) 1993, 1994, Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This file is based on work by George Kyriazis.
 */

#include "ray.h"
#include "proto.h"
#include "extern.h"

/* return a background gradient color */
color
bgcolor(struct ray *r)
{
    color           c;
    Flt             factor;



    /* if it is above the appropriate axis just give a grey value */
    /* if below give a gradually lighter color */
    factor = vdot(bggradient, r->dir);

    if (factor >= -EPSILON)
	c = background_color;
    else {
	c.r = background_color.r * (1 - 0.8 * factor);
	c.g = background_color.g * (1 - 0.8 * factor);
	c.b = background_color.b * (1 - 0.8 * factor);
    }

    c = normcol(c);

    return c;
}

/* add fog to ray */
PUBLIC color
fogsources(struct ray *r, Flt endpoint)
{
    struct light_data *l;
    object         *op;
    int             light_index;
    color           col;
    Flt             d0sq,
                    t0,
                    intensity,
                    d0;
    Flt             dirfact;

    vector          pminl,
                    closepoint,
                    closedir,
                    lorg;

    setcolor(col, 0, 0, 0);

    /* Walk the light list. */
    for (light_index = 0; light_index < light_list_len; light_index++) {
	op = light_list[light_index];
	l = op->data.light;

	if (l->type != L_FOGSOURCE)
	    continue;


	if (op->speed) {
	    /* incorporate the lightsource's speed into the calculation */
	    svproduct(lorg, Time, *op->speed);
	    vadd(lorg, lorg, l->org);
	} else
	    lorg = l->org;

	vsub(pminl, r->pos, lorg);
	t0 = -vdot(pminl, r->dir);

	dirfact = 1;
	svproduct(closepoint, t0, r->dir);
	vadd(closepoint, pminl, closepoint);
	if (l->tightness) {
	    norm(closedir, closepoint);
	    dirfact = vdot(closedir, l->direction);
	    if (dirfact < EPSILON) {
		continue;
	    }
	    dirfact = pow(dirfact, l->tightness);
	}
	d0sq = vdot(closepoint, closepoint);
	d0 = sqrt(d0sq);
	intensity = (atan((endpoint - t0) / d0) - atan(-t0 / d0)) /
	    ((d0 + l->radius) * pow((1 + d0), l->attenuation_exp));
	col = color_add(col, scolorproduct(intensity * dirfact, l->lcolor));
	continue;

    }
    return col;
}
