// A scene I originally made for PoV 1.0, converted to Rayce.

// Since the BSP in rayce likes boxes for bounding, I've adapted the BVs

/*  This is ezprezzo.pov, a datafile for rendering an
    espressomaker. The silly name was chosen to avoid
    confusion: It is not the NFF dataset by Todd Elvins, it
    is not the espresso by Lex van der Sluijs (called S-presso)
    Because my computer is rather slow, I have taken great
    pains to make it as fast possible.

    It was made by

        Han-Wen Nienhuys

    If you have suggestions, or modifications, please
    let me know, I am reachable by e-mail at

        hanwen@stack.urc.tue.nl

    or

        wsadjw@urc.tue.nl

    (don't forget to mention your mail is for me; it's not my account)
    or by snailmail at

        Dommelseweg 1a
        5581 VA Waalre, Holland.

*/


#include "standard.inc"
#include "colors.inc"
#include "shapes.inc"


#define Shiny = texture { specular 1.0 roughness 0.01 }
#define Dull = texture { specular 0.5 roughness 0.15   }

/****************/
/**** CAMERA ****/
/****************/

#define overview = camera {
    location <10, 40, -50>
    direction <0, 3.5, 0>
    aspect 4/3
    sky y
    look_at 4 * y
}

#define zoomin = camera {
    location  <15, 23, -40>
    direction <0, 2.8, 0>
    aspect 4/3
    sky y
    look_at <0, 8.5, 0>
    fov 30
}

camera { zoomin }

options {
    iterations 1
    bunching 5
}

/****************/
/**** LIGHTS ****/
/****************/

#define brightlight = color red 1.5 green 1.4 blue 0.0
// VERY bright yellow

object {
    light_source {
        <4, 30, 0>
        color brightlight
        spotlight
        point_at y
// radius 22        
// falloff 34        
//        tightness 0.26        
        tightness 1
    }
}

object {
    light_source {
        <10, 20, 0>
        color White
        spotlight
        point_at <2, 3, 0>
// radius 24        
// falloff 34        
// tightness 0.23        
        tightness 1
        rotate <0, 10, 0>
    }
}

/**************/
/*** ACTION ***/
/**************/

/* modified chrome texture
   the diffuse value is kept low otherwise,
   the spoon looks like a yellow blob
*/

#define Spoon_Texture = texture {
    microfacet Blinn
    color LightGray
    ambient 0.23
    diffuse 0.10
    reflection 0.9
    brilliance 8.0
    specular 0.8
    roughness 0.01
    refraction 0.0
    refraction 0.0
}


#define Coffee_Text =
texture {
    microfacet Blinn
    color Black
    diffuse 0.3
    specular 1.0 roughness 0.01
    brilliance 5
// dents 0.7    
    scale <0.5, 0.2, 0.5>
    reflection 0.5
    refraction 0.0
}

#define Sugar_Text =
texture {
    microfacet Blinn
    color Quartz
    specular 1.0
    roughness 0.01
// metallic    
    ambient 0.4
    diffuse 0.5
    brilliance 2
    reflection 0.1          /* needed for contrast */
// wrinkles 0.8            /* grainy surface */    
    scale <0.12, 0.12, 0.12>
    refraction 0.0
}

#define Brown_Ceramic = texture {
    microfacet Blinn
    colour Brown
    specular 1.0
    roughness 0.01
    reflection color rgb 0.2 * <1, 1, 1>
    brilliance 3.0
    diffuse 0.3
    ambient 0.3
    refraction 0.0
}

#define Cup_Texture = texture {
    Brown_Ceramic
}

/* metallic texture */
#define Gray_Metal =
texture {
    microfacet Blinn
    colour Silver
    specular 1.0
    roughness 0.01
// metallic    
    brilliance 5.0
    diffuse 0.7
    ambient 0.3
    reflection color rgb 0.3 * <1, 1, 1>
    refraction 0.0
}

#define Espresso_Metal = texture { Gray_Metal }

#define Black_plastic = texture {
    specular color Grey50
    colour Black
    microfacet Phong
    roughness 0.15
    brilliance 0.3
    diffuse 0.4
    ambient 0.6                 /* ambient black ?! [yup :) ] */
    refraction 0.0
    reflection 0.0
}

/**************************** OBJECTS ****************************************/
/* ESPRESSO MACHINE */


/*
 * handle, points at the -Z axis
 */

#define handle =       /* handle */
composite {
    /* metal part of the handle */
    object {
        intersection {
            object { QCone_Z scale <1, 1, 15> }
            box { <-1, -1, -15>,< 1, 1, -8.5> }
        }
    }
    /* black cap */
    object {
        Disk_Z
        texture {
            Black_plastic
        }
        bounded_by { object { Cube } }
        scale <1, 1, 0.5>
        translate <0, 0, -15>
    }
    bounded_by { box { <1, 1, -15>,< -1, -1, -8> } }
    
    translate <0, 0, 8.5>
}

/*
    middle part. Dimensions:
    height: 7.95
    bottom radius: 3.858
    top radius: 3.154218
*/

#define espart =           /* middle part */
composite {
    object {
        union {
            intersection {
                intersection {  /* body */
                    object         { QCone_Y scale <4.3, 48.6, 4.3> }
                    object   { QCone_Y scale <4.2, 48.6, 4.2> inverse }
                    plane { y, -35.65 }
                    plane { -y, 43.6 }
                    translate <0, 43.6, 0>
                }
                intersection {  /* take a bite out of it */
                    plane { <0, -1, -1>, 0 }
                    plane { <0, -1, 1>, 0 }
                    plane { -x, 0 }
                    translate 7 * y
                    inverse
                }
            }
            
            /* ...; it points at +X axis. */
            intersection {
                plane { <1, -1, -1>, 0 }
                plane { <1, -1, 1>, 0 }
                intersection {
                    plane { <1, -1, -1>, -0.2 }
                    plane { <1, -1, 1>, -0.2 }
                    inverse
                }
                plane { y, 1 }
                plane { -x, 0.5 }
                translate <3.15, 7, 0>
            }
        }
        bounded_by { box { <-4, 0, -4>, <4, 8, 4> } }
    }
    object {
        handle
        translate <0, 5, -2.6>
    }
}

/*
    Bottom part; this normally contains water. Dimensions:

    height: 5.00
    top radius: 3.858
    bottom radius: 4.300
*/

#define esbot = composite {             /* bottom part */
    object {
        intersection {      /* body */
            object       { QCone_Y scale <4.3, 48.6, 4.3> }
            object             { QCone_Y scale <4.2, 48.6, 4.2> inverse }
            box { <-4.3, -48.6, -4.3>, <4.3, -43.7, 4.3> }
            plane { -y, 48.6 }
            translate <0, 48.6, 0>
        }
        
    }
    object {            /* safety valve */
        intersection {
            object { Hexagon scale <0.4, 0.5, 0.5> }
            object             { Cylinder_X scale <1, 0.3, 0.3> inverse }
        }
        bounded_by {
            sphere { <0>, 1.5 }
        }
        translate <4.1, 3.5, 0>
    }
}

/*
 * the lid on top of the espressomaker.
 */
#define escap =
composite {
    object {
        intersection {  /* body */
            plane { -y, 0 }
            sphere { <0, -1.3185, 0>, 3.4185 }
// sphere { <0, -1.3185, 0>, 3.3 inverse }            
        }
    }
    
    composite {     /* the little knob on top */
        
        /* the black part */
        intersection {
            sphere { <0, 2.85, 0>, 0.75 }
            object       {
                Cylinder_Y scale <0.4, 1, 0.4> inverse
            }
            texture {
                Black_plastic
            }
        }
        
        /* metal core */
        intersection {
            sphere { <0, 2.85, 0>, 0.75 }
            object { Cylinder_Y scale <0.4, 1, 0.4> }
        }
    }
}

/* glue everything together */

#define espressomachine =
composite {
    object     {
        espart
        translate 5 * y
    }
    object   {
        escap
        translate <0, 13, 0>
    }
    object {
        esbot
        translate <0>
    }
    texture { Espresso_Metal }
}

/***********/
/*** CUP ***/
/***********/

#define cupbody =                /* body */
composite {
    object {
        intersection {
            object {            /* a Cylinder */
                Y_Disk
                scale <2.5, 5.5, 2.5>
            }
            object {            /* make it hollow */
                Cylinder_Y scale <2.1, 5.5, 2.1>
                inverse
            }
            
        }
        bounded_by { box { <-2.5, 0, -2.5>,< 2.5, 5.5, 2.5> } }
        translate <0, 1.1, 0>
    }
    object {
        intersection {                /* small stand */
            object { QCone_Y scale <2, 4.1, 2> }
            plane { y, 4.1 }
            plane { -y, -3 }
            translate <0, -3, 0>
        }
        
        bounded_by { box { <-2, 0 , -2>, <2, 1.1, 2> } }
    }
    
}

#define saucer =
object {
    intersection {
        intersection {
            sphere { <0, 4.85, 0>, 6.8 }
            plane { -y, 0 }
        }
        intersection {
            sphere { <0, 7.58, 0>, 8.47 }
            plane { -y, -0.6 }
            inverse
        }
        intersection {            /* place for the cup */
            plane { -y, -0.4 }
            object { Cylinder_Y scale <2.1, 1, 2.1> }
            inverse
        }
    }
}


#define ear =
object {
    intersection {
        torus { 1.5, 0.3
            translate -1.2 * x
            scale <0.9, 1, 1>
            rotate 90*x
            sturm
        }
        plane { x, 0 }
    }
    bounded_by { box { <-3, 1.8, -0.3>, <0, -1.8, 0.3> } }
}

/* The hub on top of the cup
 */
#define cupring = torus { 2.3, 0.2 sturm }

/****************/
/* PARAFERNALIA */
/****************/

#define Coffee =               /* YESSS! Coffee, yummie ! */
object {
    plane { y, 0 }
    clipped_by {
        sphere { <0>, 2.2 }
    }
    
    texture {
        Coffee_Text
    }
}


#define spoon =
composite {
    box { <-5, -0.2, -0.3>, <0, 0, 0.3> }
    box { <0, -0.2, -0.3>, <0.5, 0, 0.3> rotate <0, 0, -30> }
    intersection {
        sphere { <0>, 1.5 }
        sphere { <0>, 1.4 inverse }
        plane { y, -0.5 }
        scale <0.75, 0.5, 0.75>
        translate 1.14 * x
    }
    
    translate <0, 0.2, 0>
    rotate 3 * z
    texture {
        Spoon_Texture
    }
    
    bounded_by { box { <-5, -1.5, -2>, <4, 1, 2> } }
}

#define sugarcube =            /* a sugarcube */
object {
    object {
        Cube
        scale <1, 0.5, 0.75>
        translate <0, 0.5, 0>
    }
    texture {
        Sugar_Text
    }
}

/*****************************************************************************/

#define cup =
composite {
    object {
        cupbody
    }
    object {
        ear
        translate <-2.5, 3.3, 0>
    }
    object {
        cupring
        translate <0, 6.6, 0>
    }
}


#define cupandsaucer =
composite {
    object  {
        cup
        translate <0, 0.4, 0>
    }
    object { saucer }
    composite {
        object {
            sugarcube
            translate <0, 0.4, -3.3>
        }
        object {
            sugarcube
            rotate <0, 20, 0>
            translate <0, 0.4, -3.3>
            rotate <0, 45, 0>
        }
        rotate <0, -55, 0>
    }
    object {
        spoon
        rotate <0, 0, -10>
        rotate <0, 90, 0>
        translate <-4, 1.3, 0>
        rotate <0, -50, 0>
    }
    texture { Cup_Texture }
}

/*****************************************************************************/
/* now, build the scene */
object {                             /* the Holy machine */
    espressomachine
    rotate <0, 20, 0>
    translate <-6, 0, 5>
}

object {                             /* cup and thingies */
    cupandsaucer
    translate <6, 0, 2>
}

object {                                /* floor */
    plane { y, 0 }
    texture {
        colour Magenta
        ambient 0.16
        diffuse 0.84
        reflection 0.0
        refraction 0.0
    }
}
