/*
 * main.c -- main() and its belongings.
 *
 * (c) 1993, 1994 by Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>
 *
 * based on work originally by George Kyriazis, 1988.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __GNUC__
#include <unistd.h>
#endif
#include <string.h>

#include	"ray.h"
#include	"proto.h"
#include 	"extern.h"

/* proto */
PRIVATE void    usage(void);
PRIVATE void    notice(void);


PUBLIC int
main(int argc, char *argv[])
{
    extern char    *optarg;
    char            infn[100] = "input.r",
                    outfn[100] = "output.tga";
    char            c;

    /* print the (c) stuff */
    notice();

    /* no commandline options? */
    if (argc == 1)
	usage();

    /* set some global vars */
    set_defaults();
    initialize();
    /* scan commandline */
    while ((c = getopt(argc, argv, "l:d:h:w:I:o:i:sxtch")) != EOF) {
	switch (c) {
	case 'c':		/* continue a previously interrupted picture */
	    continue_flag = TRUE;
	    break;

	case 'd':		/* turn on debugging */

#ifdef DEBUG
	    debug_options = atoi(optarg);
	    if (!debug_options)
		debug_options = ~0;
#else
	    errormsg("Rayce wasn't compiled with DEBUG defined");
#endif				/* DEBUG */
	    break;

	case 's':
	    silent_mode = TRUE;
	    break;
	case 'h':		/* horizontal resolution */
	    yres = atoi(optarg);
	    break;
	case 'w':		/* vertical resolution */
	    xres = atoi(optarg);
	    break;
	case 'x':		/* disable keypress-exit */
	    keyboard_exit = TRUE;
	    break;
	case 't':		/* for testing 'n debugging */
	    test_flag = TRUE;
	    break;
	case 'l':		/* library path */

	    libpath_len++;
	    library_path = realloc(library_path, libpath_len);
	    library_path[libpath_len - 1] = xstrdup(optarg);

	    break;
	case 'I':		/* number of frames, "iterations" */
	    iter_override = atoi(optarg);
	    tries = iter_override;
	    break;

	case 'o':		/* output file */
	    strcpy(outfn, optarg);
	    break;
	case 'i':		/* inputfile */
	    strcpy(infn, optarg);
	    break;
	case '?':
	    usage();
	    break;
	default:
	    assert(FALSE);
	}
    }


    /* initialize any machine dependent stuff */
    init_machine();

    /* parse the input */
    readfile(infn);
    precompute_globals();
    if (debug_options & DEBUGINT)
	print_globals();

    start_stats();

    /* and do the tracing */
    raytrace(outfn);

    exit(0);
}

/* print help screen */
PRIVATE void
usage(void)
{
    printf(
	      "Rayce <options>\n"
	      "-I #        number of frames\n"
	      "-i <file>   input\n"
	      "-o <file>   output\n"
	      "-h #        height\n"
	      "-w #        width\n"
	      "-x          disable exit\n"
	      "-c          continue trace\n"
	      "-d #        debugging (needs -DDEBUG)\n"
	      "-t          testing\n"
	      "-l path     library\n"
	      "  1 = tokenizer, 2 = parser, 4 = runtime warnings, \n"
	      "  8 = print interior, 16 = print defines\n"
	      "\n"
	      "  Rayce was compiled with DEBUG %sdefined\n",

#ifdef DEBUG
	      ""
#else
	      "not "
#endif				/* DEBUG */

    );
    exit(0);
}

PRIVATE void
notice(void)
{
    printf("This is Rayce version 2.8\n"
	   "Rayce is (c) 1993, 1994 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>\n"
	   "  It is based on \"Ray\" by George Kyriazis <kyriazis@turing.cs.rpi.edu>\n"
	   "  Rayce is released under the terms of the GNU General Public License,\n"
	   "  it has no warranty what so ever. For more details: see the file \"COPYING\"\n"
	   "\n"
	   "Enjoy!\n\n");

    return;
}
