/*
 * message.c -- input/output of messages, debugging dumps
 *
 * (c) 1994 by Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdarg.h>
#include <string.h>
#include	"ray.h"
#include	"proto.h"
#include 	"extern.h"

#define 	INDENTAMOUNT 2

PUBLIC void    *
xmalloc(size_t s, char *type)
{
    void           *ptr;

    ptr = malloc(s);
    if (!ptr)
	if (s)
	    errormsg("Out of memory while allocating %s", type);
    return ptr;
}

PUBLIC char    *
xstrdup(char *s)
{
    char           *ptr;

    ptr = malloc((strlen(s) + 1) * sizeof(char));

    if (!ptr)
	errormsg("Out of memory while duplicating string");
    strcpy(ptr, s);
    return ptr;
}

PUBLIC void    *
xrealloc(void *ptr, size_t s, char *type)
{
    void           *p;

    p = realloc(ptr, s);

    if (p == NULL)
	if (s)
	    errormsg("Out of memory while reallocating %s", type);
    return p;
}

/* prints an error message with linenumber to stdout and then dies */
PUBLIC void
errormsg(char *format,...)
{
    char            s1[100],
                    s2[100];
    va_list         arglist;

    if (inputlines > 0)		/* line number appropriate? */
	sprintf(s1, "\n%s: %d: ", this_file, inputlines);
    else
	sprintf(s1, "\nerror: ");

    va_start(arglist, format);
    vsprintf(s2, format, arglist);
    va_end(arglist);

    fprintf(error_out, "%s%s\n", s1, s2);

    exit(1);
}

/* prints a warning with linenumber to stdout */
PUBLIC void
warning(char *format,...)
{
    char            s1[100],
                    s2[100];
    va_list         arglist;

    if (inputlines > 0)		/* line number appropriate? */
	sprintf(s1, "\n%s: %d: warning: ", this_file, inputlines);
    else
	sprintf(s1, "\nwarning: ");


    va_start(arglist, format);
    vsprintf(s2, format, arglist);
    va_end(arglist);

    fprintf(error_out, "%s%s\n", s1, s2);
}

PRIVATE int     indent = 0;


/* a printf with indent before each line */
PUBLIC void
dprintf(char *format,...)
{
    char            s[1000],
                   *cp;
    va_list         arglist;
    int             i;

    if (trace_start) {
	va_start(arglist, format);
	vfprintf(error_out, format, arglist);
	va_end(arglist);
	return;
    }
    va_start(arglist, format);
    vsprintf(s, format, arglist);
    va_end(arglist);

    for (cp = s; *cp; cp++)
	switch (*cp) {
	case '\n':
	    putc('\n', error_out);

	    for (i = 0; i < indent; i++)
		putc(' ', error_out);
	    break;
	case '<':
	case '(':
	case '{':
	case '[':
	    indent += INDENTAMOUNT;
	    putc(*cp, error_out);

	    break;
	case ']':
	case '}':
	case '>':
	case ')':
	    indent -= INDENTAMOUNT;
	    putc(*cp, error_out);
	    break;
	default:
	    putc(*cp, error_out);
	    break;
	}
}

/* open file, searching library paths */
PUBLIC FILE    *
open_file(char *name, char *mode)
{
    FILE           *f;
    char            s[1000];
    int             i;

    strcpy(s, name);
    f = fopen(s, mode);
    for (i = 0; f == NULL && i < libpath_len; i++) {
	strcpy(s, library_path[i]);
	strcat(s, PATHSEP);
	strcat(s, name);
	f = fopen(s, mode);
    }

    if (f == NULL) {
	errormsg("Cannot find file %s", name);
	return NULL;
    } else {
	printf("[%s ", s);
	return f;
    }
}

PUBLIC void
close_file(FILE * f)
{
    printf("]");		/* show that we closing */
    fclose(f);
}
