
/*
 * optimise.c -- optimisation utility routines
 *
 * (c) 1994 by Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "ray.h"
#include "proto.h"


/* add one object to the  optimisation list */
PUBLIC void
add_to_optilist(struct optilist *thelist, object *op)
{
    int             l;
    vector          diff;

    if (thelist->len >= thelist->max) {
	thelist->max = 2 * thelist->max + 1;
	thelist->list = xrealloc(thelist->list, thelist->max * sizeof(object *), "optimisation list");
    }
    l = thelist->len++;
    thelist->list[l] = op;

    /*
     * svproduct(thelist->mean, (Flt) l / (Flt) (l + 1), thelist->mean); vadd(diff, op->bbox[1], op->bbox[0]);
     * svproduct(diff, 0.5 / (Flt) (l + 1), diff); vadd(thelist->mean, diff, thelist->mean);
     */

    if (op->bbox[0].x < thelist->bmin.x)
	thelist->bmin.x = op->bbox[0].x;
    if (op->bbox[0].y < thelist->bmin.y)
	thelist->bmin.y = op->bbox[0].y;
    if (op->bbox[0].z < thelist->bmin.z)
	thelist->bmin.z = op->bbox[0].z;
    if (op->bbox[1].x > thelist->bmax.x)
	thelist->bmax.x = op->bbox[1].x;
    if (op->bbox[1].y > thelist->bmax.y)
	thelist->bmax.y = op->bbox[1].y;
    if (op->bbox[1].z > thelist->bmax.z)
	thelist->bmax.z = op->bbox[1].z;
}

/* recurse into o */
PRIVATE void
do_build_list(object *o, struct optilist *thelist)
{
    struct composite_data *c;
    object         *op;

    if (o->type != COMPOSITE)
	return;

    for (op = o->data.composite->contents; op != NULL; op = op->next) {
	switch (op->type) {

	case COMPOSITE:
	    c = op->data.composite;
	    if (c->optitype == NONE && op->clip == NULL)
		do_build_list(op, thelist);
	    else
		add_to_optilist(thelist, op);
	    break;

	    /* add this primitive */
	default:
	    if (op->flags & O_INFINITE)
		warning("Cannot autobound infinite object %s", op->methods->name);
	    if (!(op->flags & O_SMALL))
		add_to_optilist(thelist, op);
	}
    }
}

/* build a list from o, and everything in it */
PUBLIC void
build_list(object *o, struct optilist *thelist)
{
    thelist->len = 0;
    thelist->list = NULL;
    thelist->max = 0;
    /*
     * setvector(thelist->mean, 0, 0, 0);
     */
    setvector(thelist->bmin, INFTY, INFTY, INFTY);
    setvector(thelist->bmax, -INFTY, -INFTY, -INFTY);
    do_build_list(o, thelist);
}

/* zap */
PUBLIC void
free_optimisation_list(struct optilist *l)
{
    free(l->list);
    free(l);
}

/* reduce memory used by this one. */
PUBLIC void
clean_optimisation_list(struct optilist *l)
{
    if (l->len) {
	l->list = xrealloc(l->list, l->len * sizeof(object *), "optilist");

	l->max = l->len;
    }
}

/* this is really standard */
PUBLIC struct optilist *
get_new_optimisation_list(void)
{
    struct optilist *thelist;

    thelist = ALLOC(struct optilist);



    thelist->len = 0;
    thelist->list = NULL;
    thelist->max = 0;

    /* setvector(thelist->mean, 0, 0, 0); */
    setvector(thelist->bmin, INFTY, INFTY, INFTY);
    setvector(thelist->bmax, -INFTY, -INFTY, -INFTY);

    return thelist;
}

/* and a debugging tool */
PUBLIC void
print_optimisation_list(struct optilist *thelist)
{
#ifdef DEBUG
    int             i;

    dprintf("len %d\n", thelist->len);
    for (i = 0; i < thelist->len; i++)
	short_print_object(thelist->list[i]);

#endif				/* DEBUG */
}
