/*
  paperclip, mimicked from the Cook/Porter/Carpenter article. if you
  remove all the "sturm" keywords, and use the "zoom" camera, it makes
  a great demo of the quartics numerical inaccuracies.

  It looks pretty good at 640x480x20 iterations, but you could do with
  less rays per pixel, especially if you use a higher resolution

  HWN
*/


#include "standard.inc"
#include "colors.inc"
#include "shapes.inc"

#define final = camera {
    location <-3, 7, -20>
    look_at <0>
    sky y
    fov 20
    aspect 1.0
}

#define zoom = camera {
    location <-3, 6, -10>
    look_at <0, 1.5, 0>
    sky y
    fov 18
}

camera {final }

options {
    background y color Grey20
    iterations 16
    tolerance 0.007
    antialias Gauss_filter
    bunching 5
}

#define Wood_Finish = texture {
    reflection color Grey80
    reflect_angle 1.5
    diffuse color Brown diffuse 0.45
    ambient color Brown ambient 0.05
}

#define table = object {
    polygon { <-20, 0, 7>, <7, 0, 7>, <7, 0, -12>, <-20, 0, -12> }
    texture { Wood_Finish }
}

#define paperclip = composite {
    object {
        intersection {
            torus { 0.3, 0.05 sturm rotate <90, 0, 0> }
            plane { y, 0 }
        }
    }
    cylinder { <-0.3, 0, 0>, <-0.3, 2, 0>, 0.05 }
    cylinder { <0.3, 0, 0>, <0.3, 1.25, 0>, 0.05 }
    intersection {
        torus { 0.25, 0.05 sturm rotate <90, 0, 0> }
        plane { -y, 0 }
        translate <-0.05, 2, 0>
    }
    cylinder { <0.2, 0.5, 0>, <0.2, 2.0, 0>, 0.05 }
    object {
        intersection {
            torus { 0.2, 0.05 sturm rotate <90, 0, 0> }
            plane { y, 0 }
            translate <0, 0.5, 0>
        }
    }
    cylinder { <-0.2, 0.5, 0>, <-0.2, 1.25, 0>, 0.05 }
    texture {
        specular color rgb 0.5 * <color White>
        roughness 0.15
        microfacet Blinn
        diffuse color White diffuse 0.35
        ambient color White ambient 0.07
    }
    translate <0, 0.35, 0>
}

object { paperclip }

object { table }

/* high noon */
object {
    light_source {
        <-6, 13, -13>
        color Gray40
        radius 2
    }
}

/* casting low shadow */
object {
    light_source {
        <-3, 10, 40>
        color White
        radius 3
    }
}
